import os
import logging
import logging.config


# Logging Config
# More on Logging Configuration
# https://docs.python.org/3/library/logging.config.html
# Setting up a config

LOGGING_DEFAULT_CONFIG = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "default": {
            "format": "%(asctime)s - %(name)s - %(levelname)s - %(funcName)s:%(lineno)d - %(message)s",
            "datefmt": "%Y-%m-%d %H:%M:%S",
        },
        "simple": {"format": "%(message)s"},
    },
    "root": {"level": "INFO"},
}


def configure_logger(
    logger=None, cfg=None, log_file=None, console=True, log_level="DEBUG"
):
    """Function to setup configurations of logger through function.
    The individual arguments of `log_file`, `console`, `log_level` will overwrite the ones in cfg.
    Parameters
    ----------
            logger:
                    Predefined logger object if present. If None a ew logger object will be created from root.
            cfg: dict()
                    Configuration of the logging to be implemented by default
            log_file: str
                    Path to the log file for logs to be stored
            console: bool
                    To include a console handler(logs printing in console)
            log_level: str
                    One of `["INFO","DEBUG","WARNING","ERROR","CRITICAL"]`
                    default - `"DEBUG"`
    Returns
    -------
    logging.Logger
    """
    if not cfg:
        logging.config.dictConfig(LOGGING_DEFAULT_CONFIG)
    else:
        logging.config.dictConfig(cfg)

    logger = logger or logging.getLogger()

    if log_file or console:
        for hdlr in logger.handlers:
            logger.removeHandler(hdlr)

        if log_file:
            fh = logging.FileHandler(log_file)
            fh.setLevel(getattr(logging, log_level))
            logger.addHandler(fh)

        if console:
            sh = logging.StreamHandler()
            sh.setLevel(getattr(logging, log_level))
            logger.addHandler(sh)

    return logger





import argparse
import os
import pickle

import numpy as np
import pandas as pd
from sklearn.metrics import mean_absolute_error, mean_squared_error

from HousingPriceSP.logger import configure_logger


def get_data(path):
    prepared = pd.read_csv(path + "/test_x.csv")
    lables = pd.read_csv(path + "/test_y.csv")
    lables = lables.values.ravel()
    return prepared, lables


def score(model, x_test, y_test):
    loadModel = pickle.load(open(model, "rb"))
    predictions = loadModel.predict(x_test)
    mse = mean_squared_error(y_test, predictions)
    rmse = np.sqrt(mse)
    mae = mean_absolute_error(y_test, predictions)
    return mse, rmse, mae


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("--log-level", type=str, default="DEBUG")
    parser.add_argument("--no-console-log", action="store_true")
    parser.add_argument("--log-path", type=str, default=os.getcwd() + "/logs/logs.log")
    parser.add_argument(
        "--datapath", help="path to store the dataset", type=str, default="/data/raw"
    )
    parser.add_argument(
        "--dataprocessed",
        help="path to store the dataset",
        type=str,
        default="/data/processed",
    )
    parser.add_argument(
        "--artifacts", help="path to store artifacts", type=str, default="/artifacts"
    )

    args = parser.parse_args()
    logger = configure_logger(
        log_level=args.log_level,
        log_file=args.log_path,
        console=not args.no_console_log,
    )

    repo_path = os.getcwd()
    data_path = repo_path + args.dataprocessed
    model_path = repo_path + args.artifacts

    x_test, y_test = get_data(data_path)

    mse, rmse, mae = score(os.path.join(model_path, "LinearRegression"), x_test, y_test)
    logger.debug(f"Linear Regression -> mse= {mse} , rmse= {rmse}, mae= {mae}")
    mse, rmse, mae = score(
        os.path.join(model_path, "DecisionTreeRegression"), x_test, y_test
    )
    logger.debug(f"Decision Tree Regression -> mse= {mse} , rmse= {rmse}, mae= {mae}")
    mse, rmse, mae = score(
        os.path.join(model_path, "RandomForestRegressor_RandomSearch"),
        x_test,
        y_test,
    )
    logger.debug(
        f"RandomForestRegressor RandomSearch -> mse= {mse} , rmse= {rmse}, mae= {mae}"
    )
    mse, rmse, mae = score(
        os.path.join(model_path, "randomForestRegressor_GridSearch"), x_test, y_test
    )
    logger.debug(
        f"RandomForestRegressor GridSearch -> mse= {mse} , rmse= {rmse}, mae= {mae}"
    )


import argparse
import os
import pickle

import pandas as pd
from scipy.stats import randint
from sklearn.ensemble import RandomForestRegressor
from sklearn.linear_model import LinearRegression
from sklearn.model_selection import GridSearchCV, RandomizedSearchCV
from sklearn.tree import DecisionTreeRegressor

from HousingPriceSP.logger import configure_logger


def get_data(path):
    prepared = pd.read_csv(path + "/train_x.csv")
    lables = pd.read_csv(path + "/train_y.csv")
    lables = lables.values.ravel()
    return prepared, lables


def train(housing_prepared, housing_labels):
    lin_reg = LinearRegression()
    lin_reg.fit(housing_prepared, housing_labels)

    tree_reg = DecisionTreeRegressor(random_state=42)
    tree_reg.fit(housing_prepared, housing_labels)

    param_distribs = {
        "n_estimators": randint(low=1, high=200),
        "max_features": randint(low=1, high=8),
    }

    forest_reg = RandomForestRegressor(random_state=42)
    rnd_search = RandomizedSearchCV(
        forest_reg,
        param_distributions=param_distribs,
        n_iter=10,
        cv=5,
        scoring="neg_mean_squared_error",
        random_state=42,
    )
    rnd_search.fit(housing_prepared, housing_labels)

    param_grid = [
        # try 12 (3×4) combinations of hyperparameters
        {"n_estimators": [3, 10, 30], "max_features": [2, 4, 6, 8]},
        # then try 6 (2×3) combinations with bootstrap set as False
        {"bootstrap": [False], "n_estimators": [3, 10], "max_features": [2, 3, 4]},
    ]

    forest_reg = RandomForestRegressor(random_state=42)
    # train across 5 folds, that's a total of (12+6)*5=90 rounds of training
    grid_search = GridSearchCV(
        forest_reg,
        param_grid,
        cv=5,
        scoring="neg_mean_squared_error",
        return_train_score=True,
    )
    grid_search.fit(housing_prepared, housing_labels)

    return lin_reg, tree_reg, rnd_search, grid_search


def dump_model(model, output_path, filname):
    with open(os.path.join(output_path, filname), "wb") as p:
        pickle.dump(model, p)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("--log-level", type=str, default="DEBUG")
    parser.add_argument("--no-console-log", action="store_true")
    parser.add_argument("--log-path", type=str, default=os.getcwd() + "/logs/logs.log")
    parser.add_argument(
        "--datapath", help="path to store the dataset", type=str, default="/data/raw"
    )
    parser.add_argument(
        "--dataprocessed",
        help="path to store the dataset",
        type=str,
        default="/data/processed",
    )
    parser.add_argument(
        "--artifacts", help="path to store artifacts", type=str, default="/artifacts"
    )

    args = parser.parse_args()
    logger = configure_logger(
        log_level=args.log_level,
        log_file=args.log_path,
        console=not args.no_console_log,
    )

    repo_path = os.getcwd()
    input_path = repo_path + args.dataprocessed
    output_path = repo_path + args.artifacts

    x, y = get_data(input_path)
    logger.debug(f"Fetched training data from {input_path}.")
    logger.debug("Going to Run models...")
    lin_reg, tree_reg, forest_reg, grid_search = train(x, y)
    logger.debug("Training completed")
    models_name = [
        "LinearRegression",
        "DecisionTreeRegression",
        "RandomForestRegressor_RandomSearch",
        "randomForestRegressor_GridSearch",
    ]
    dump_model(lin_reg, output_path, models_name[0])
    dump_model(tree_reg, output_path, models_name[1])
    dump_model(forest_reg, output_path, models_name[2])
    dump_model(grid_search, output_path, models_name[3])
    logger.debug(f"All Models are saved at {output_path}.")


# setup.py placed at root directory
from setuptools import setup

setup(
    name='HousingPriceSP',
    version='1.1.0',
    author='Prakash Singh',
    description='Package to Predict housing Price!',
    long_description='This Package will help to predict housing price using regression model and using differnt parameters',
    packages=['HousingPriceSP'],
    python_requires=">=3.6"

)


import argparse
import os
import tarfile

import numpy as np
import pandas as pd
from six.moves import urllib
from sklearn.impute import SimpleImputer
from sklearn.model_selection import StratifiedShuffleSplit, train_test_split

from HousingPriceSP.logger import configure_logger

DOWNLOAD_ROOT = "https://raw.githubusercontent.com/ageron/handson-ml/master/"
HOUSING_PATH = os.path.join("data", "raw")
HOUSING_URL = DOWNLOAD_ROOT + "datasets/housing/housing.tgz"


def fetch_housing_data(housing_url=HOUSING_URL, housing_path=HOUSING_PATH):
    os.makedirs(housing_path, exist_ok=True)
    tgz_path = os.path.join(housing_path, "housing.tgz")
    urllib.request.urlretrieve(housing_url, tgz_path)
    housing_tgz = tarfile.open(tgz_path)
    housing_tgz.extractall(path=housing_path)
    housing_tgz.close()


def load_housing_data(housing_path=HOUSING_PATH):
    csv_path = os.path.join(housing_path, "housing.csv")
    return pd.read_csv(csv_path)


def income_cat_proportions(data):
    return data["income_cat"].value_counts() / len(data)


def split(housing):

    housing["income_cat"] = pd.cut(
        housing["median_income"],
        bins=[0.0, 1.5, 3.0, 4.5, 6.0, np.inf],
        labels=[1, 2, 3, 4, 5],
    )
    train_set, test_set = train_test_split(housing, test_size=0.2, random_state=42)

    split = StratifiedShuffleSplit(n_splits=1, test_size=0.2, random_state=42)
    for train_index, test_index in split.split(housing, housing["income_cat"]):
        strat_train_set = housing.loc[train_index]
        strat_test_set = housing.loc[test_index]

    compare_props = pd.DataFrame(
        {
            "Overall": income_cat_proportions(housing),
            "Stratified": income_cat_proportions(strat_test_set),
            "Random": income_cat_proportions(test_set),
        }
    ).sort_index()
    compare_props["Rand. %error"] = (
        100 * compare_props["Random"] / compare_props["Overall"] - 100
    )
    compare_props["Strat. %error"] = (
        100 * compare_props["Stratified"] / compare_props["Overall"] - 100
    )

    for set_ in (strat_train_set, strat_test_set):
        set_.drop("income_cat", axis=1, inplace=True)

    return strat_train_set, strat_test_set


def preprocess(strat_train_set):
    housing = strat_train_set.copy()
    housing.plot(kind="scatter", x="longitude", y="latitude")
    housing.plot(kind="scatter", x="longitude", y="latitude", alpha=0.1)

    corr_matrix = housing.corr()
    corr_matrix["median_house_value"].sort_values(ascending=False)
    housing["rooms_per_household"] = housing["total_rooms"] / housing["households"]
    housing["bedrooms_per_room"] = housing["total_bedrooms"] / housing["total_rooms"]
    housing["population_per_household"] = housing["population"] / housing["households"]

    housing = strat_train_set.drop(
        "median_house_value", axis=1
    )  # drop labels for training set
    housing_labels = strat_train_set["median_house_value"].copy()

    imputer = SimpleImputer(strategy="median")

    housing_num = housing.drop("ocean_proximity", axis=1)

    imputer.fit(housing_num)
    X = imputer.transform(housing_num)

    housing_tr = pd.DataFrame(X, columns=housing_num.columns, index=housing.index)
    housing_tr["rooms_per_household"] = (
        housing_tr["total_rooms"] / housing_tr["households"]
    )
    housing_tr["bedrooms_per_room"] = (
        housing_tr["total_bedrooms"] / housing_tr["total_rooms"]
    )
    housing_tr["population_per_household"] = (
        housing_tr["population"] / housing_tr["households"]
    )

    housing_cat = housing[["ocean_proximity"]]
    housing_prepared = housing_tr.join(pd.get_dummies(housing_cat, drop_first=True))
    return housing_prepared, housing_labels


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("--log-level", type=str, default="DEBUG")
    parser.add_argument("--no-console-log", action="store_true")
    parser.add_argument("--log-path", type=str, default=os.getcwd() + "/logs/logs.log")
    parser.add_argument(
        "--datapath", help="path to store the dataset", type=str, default="/data/raw"
    )
    parser.add_argument(
        "--dataprocessed",
        help="path to store the dataset",
        type=str,
        default="/data/processed",
    )
    parser.add_argument(
        "--artifacts", help="path to store artifacts", type=str, default="artifacts"
    )

    args = parser.parse_args()
    logger = configure_logger(
        log_level=args.log_level,
        log_file=args.log_path,
        console=not args.no_console_log,
    )

    repo_path = os.getcwd()
    path = repo_path + args.datapath
    fetch_housing_data(housing_path=path)
    logger.debug(f"Fetched and saved housing data at {path}.")

    housing = load_housing_data(housing_path=path)
    logger.debug("Loaded housing data.")

    training_set, testing_set = split(housing)
    logger.debug("Splitted data into training and testing set.")

    train_x, train_y = preprocess(training_set)
    test_x, test_y = preprocess(testing_set)
    logger.debug("Preprossed housing dataset.")

    processed_path = repo_path + args.dataprocessed
    trainpath_x = os.path.join(processed_path, "train_x.csv")
    testpath_x = os.path.join(processed_path, "test_x.csv")
    trainpath_y = os.path.join(processed_path, "train_y.csv")
    testpath_y = os.path.join(processed_path, "test_y.csv")

    train_x.to_csv(trainpath_x, index=False)
    test_x.to_csv(testpath_x, index=False)
    train_y.to_csv(trainpath_y, index=False)
    test_y.to_csv(testpath_y, index=False)
    logger.debug(f"Saved preprossed train and test datasets at {processed_path}.")